#include <string>

#include <AntTweakBar.h>

class AntTWBarGroup
{
public:
	template< typename T >
	struct TypeMapping {
		enum {
			Type = TW_TYPE_UNDEF
		};
	};

	template<>
	struct TypeMapping< bool > {
		enum {
			Type = TW_TYPE_BOOLCPP
		};
	};

	template<>
	struct TypeMapping< char * > {
		enum {
			Type = TW_TYPE_CDSTRING
		};
	};

	template<>
	struct TypeMapping< float > {
		enum {
			Type = TW_TYPE_FLOAT
		};
	};

	template<>
	struct TypeMapping< int > {
		enum {
			Type = TW_TYPE_INT32
		};
	};

	typedef float Color4F[4];

	template<>
	struct TypeMapping< Color4F > {
		enum {
			Type = TW_TYPE_COLOR4F
		};
	};

protected:
	AntTWBarGroup * getParent();
	TwBar * getBar();

	const std::string getBarIdentifier();
	const std::string getQualifiedIdentifier();
	const std::string getQualifiedIdentifier( const std::string &itemName );

	const std::string getName();
	const std::string getGroupDefine();
	const std::string getNameDefine( const std::string &name );
	const std::string getIdentifier( const std::string &itemName );

	// identifier to be used for the AntTweakBar group
	const std::string getIdentifier();

public:
	template< class T >
	void addButton(const std::string &name, void (TW_CALL *callback )( T& ), T *clientData, const std::string &def = "", const std::string &internalName = "") {
		_addButton(name, (TwButtonCallback) callback, clientData, def, internalName );
	}

	void addSeparator();

	template< class T, typename V >
	void addVarCB(const std::string &name, void (TW_CALL *setCallback)( const V & , T & ), void (TW_CALL *getCallback)( V&, T & ), T *clientData, const std::string &def = "", const std::string &internalName = "") {
		_addVarCB( name, (TwType) TypeMapping< V >::Type, (TwSetVarCallback) setCallback, (TwGetVarCallback) getCallback, clientData, def, internalName );
	}

	template< typename V >
	void addVarRW(const std::string &name, V &var, const std::string &def = "", const std::string &internalName = "") {
		_addVarRW( name, (TwType) TypeMapping< V >::Type, &var, def, internalName );
	}

	void changeItem( const std::string &internalName, const std::string &def );
	void changeItemName( const std::string &internalName, const std::string &newName );

	void removeItem( const std::string &name );

	void clear();

private:
	AntTWBarGroup * const m_parent;

	std::string m_name;
	TwBar * const m_bar;	
	bool m_initialized;
	bool m_visible;
	bool m_expanded;

	void _addButton(const std::string &name, TwButtonCallback callback, void *clientData, const std::string &def, const std::string &internalName);
	void _addVarCB(const std::string &name, TwType type, TwSetVarCallback setCallback, TwGetVarCallback getCallback, void *clientData, const std::string &def, const std::string &internalName);
	void _addVarRW(const std::string &name, TwType type, void *var, const std::string &def, const std::string &internalName);

	void checkInit();

	void updateDefine();

	TwBar *createBar();

public:
	AntTWBarGroup( const std::string &name );
	AntTWBarGroup( const std::string &name, AntTWBarGroup &parent );
	AntTWBarGroup( const std::string &name, AntTWBarGroup *parent );

	virtual ~AntTWBarGroup();

	void setName( const std::string &name );
	void setVisible( bool visible );
	void setExpand( bool expand );

private:
	std::string format( const char *format, ... );

	static int s_groupCounter;
	int m_groupIndex;
};